/* ============================================================================
 * Copyright (C) 2004 - Infineon Technologies AG.
 *
 * All rights reserved.
 * ============================================================================
 *
 * ============================================================================
 *
 * This document contains proprietary information belonging to Infineon
 * Technologies AG. Passing on and copying of this document, and communication
 * of its contents is not permitted without prior written authorisation.
 *
 * ============================================================================
 */

/* ===========================================================================
 *
 * File Name    :   adslAtucPhysTable_access.c
 * Author       :   Subramani
 * Date         :   17-Jun-2004
 *
 * ===========================================================================
 *
 * Project      :   Amazon
 * Block        :   ADSL-MIBs
 *
 * ===========================================================================
 * Contents: This file contains functions for:
 *                  1) Getting the data context pointer & loop context
 *                  pointer
 *                  2) User-defined data access functions (per column) for data
 *                  in table adslAtucPhysTable
 *
 *
 * ===========================================================================
 * References: RFC 2662
 *
 */



/* ===========================================================================
 * Revision History:
 * Subramani:  24-Jun04 Changed the return types for get_adslAtucCurrOutputPwr
 * and  get_adslAtucCurrAttainableRate
 * $Log$
 * ===========================================================================
 */

/*
 * Note: this file originally auto-generated by mib2c using
 *        : mib2c.access_functions.conf,v 1.6 2003/09/11 22:48:49 rstory Exp $
 */

/*
 * ===========================================================================
 *                           INCLUDE FILES
 * ===========================================================================
*/

#include <net-snmp/net-snmp-config.h>
#include <net-snmp/net-snmp-includes.h>
#include <net-snmp/agent/net-snmp-agent-includes.h>

#include "adslAtucPhysTable_access.h"
#include "adslAtucPhysTable_enums.h"

/* column number definitions for table adslAtucPhysTable */
#include "adslAtucPhysTable_columns.h"

/*
 * ===========================================================================
 *                          GLOBAL DEFINITIONS
 * ===========================================================================
*/

// *** Global variable definitions
static u_int32 long_ret;
static char sys_info[32]= "";

/*
 * ===========================================================================
 *                          LOCAL FUNCTIONS
 * ===========================================================================
*/

/** returns the first data point within the adslAtucPhysTable table data.

    Set the my_loop_context variable to the first data point structure
    of your choice (from which you can find the next one).  This could
    be anything from the first node in a linked list, to an integer
    pointer containing the beginning of an array variable.

    Set the my_data_context variable to something to be returned to
    you later that will provide you with the data to return in a given
    row.  This could be the same pointer as what my_loop_context is
    set to, or something different.

    The put_index_data variable contains a list of snmp variable
    bindings, one for each index in your table.  Set the values of
    each appropriately according to the data matching the first row
    and return the put_index_data variable at the end of the function.
*/
netsnmp_variable_list *
adslAtucPhysTable_get_first_data_point(void **my_loop_context,
                                       void **my_data_context,
                                       netsnmp_variable_list *
                                       put_index_data,
                                       netsnmp_iterator_info *mydata)
{

   int *My_Data_Context;
    My_Data_Context = SNMP_MALLOC_TYPEDEF(int);
    if (!My_Data_Context) {
        put_index_data = NULL;
        snmp_log(LOG_ERR, "Get-First: adslAtucPhysTable: My_Data_Context malloc failed\n");
        return put_index_data;
    }

    *My_Data_Context=IFTYPE_ADSL;
    *my_data_context=My_Data_Context;

    put_index_data = adslTable_get_first_data_point(my_loop_context,
                                   my_data_context,
                                   put_index_data,
                                   mydata);

    SNMP_FREE(My_Data_Context);
    return put_index_data;
}

/** functionally the same as adslAtucPhysTable_get_first_data_point, but
   my_loop_context has already been set to a previous value and should
   be updated to the next in the list.  For example, if it was a
   linked list, you might want to cast it to your local data type and
   then return my_loop_context->next.  The my_data_context pointer
   should be set to something you need later and the indexes in
   put_index_data updated again. */
netsnmp_variable_list *
adslAtucPhysTable_get_next_data_point(void **my_loop_context,
                                      void **my_data_context,
                                      netsnmp_variable_list *
                                      put_index_data,
                                      netsnmp_iterator_info *mydata)
{

   int *My_Data_Context;
    My_Data_Context = SNMP_MALLOC_TYPEDEF(int);
    if (!My_Data_Context) {
        put_index_data = NULL;
        snmp_log(LOG_ERR, "Get-Next: adslAtucPhysTable: My_Data_Context malloc failed\n");
        return put_index_data;
    }
    *My_Data_Context=IFTYPE_ADSL;
    *my_data_context=My_Data_Context;

     put_index_data = adslTable_get_next_data_point(my_loop_context,
                                   my_data_context,
                                   put_index_data,
                                   mydata);

    SNMP_FREE(My_Data_Context);
    return put_index_data;
}

/*
 * User-defined data access functions (per column) for data in table
 * adslAtucPhysTable
 */
/*
 * NOTE:
 * - these get_ routines MUST return data that will not be freed (ie,
 *   use static variables or persistent data).  It will be copied, if
 *   needed, immediately after the get_ routine has been called.
 * - these SET routines must copy the incoming data and can not take
 *   ownership of the memory passed in by the val pointer.
 */

#if defined(HAVE_OPTIMIZED_CODE)

char  *
get_adslAtucInvData(void *data_context, size_t * ret_len,
			unsigned int var)
{
  unsigned int flags = 0;
  struct adslAtucPhysEntry adslAtucPhysStruct;
  memset(&adslAtucPhysStruct,0,sizeof(adslAtucPhysEntry));
  memset(sys_info,0,sizeof(sys_info));
  adslAtucPhysStruct.ifIndex= (*(int *)data_context);

  switch (var) {

        case COLUMN_ADSLATUCINVSERIALNUMBER:
	   flags = ATUC_PHY_SER_NUM_FLAG;
	   break;

        case COLUMN_ADSLATUCINVVENDORID:
	   flags = ATUC_PHY_VENDOR_ID_FLAG;
	   break;

        case COLUMN_ADSLATUCINVVERSIONNUMBER:
	   flags = ATUC_PHY_VER_NUM_FLAG;
	   break;

     	default:
     	   DEBUGMSGTL(("Unknown MIB variable requested!", "\n"));
     	   return NULL;
  	}
  
    SET_FLAG(&adslAtucPhysStruct.flags, flags);

#ifdef ENABLE_DRIVER_CALL
    adsl_ioctl(Fd, GET_ADSL_ATUC_PHY,(void *) &adslAtucPhysStruct);
#else
    get_adslAtucPhysTable1(Fd, GET_ADSL_ATUC_PHY,(void *) &adslAtucPhysStruct);
#endif
    if (IS_FLAG_SET(&adslAtucPhysStruct.flags, flags) == 0) 
       DEBUGMSGTL(("adslMIB-ATUC", "PhysTable: GET- %x CMV fail\n", flags));
    
    switch (flags)  {
	    
	case ATUC_PHY_SER_NUM_FLAG:
    	   strcpy(sys_info,adslAtucPhysStruct.serial_no);
    	   *ret_len = strlen(adslAtucPhysStruct.serial_no);
	   break;

	case ATUC_PHY_VENDOR_ID_FLAG:
    	   strcpy(sys_info,adslAtucPhysStruct.vendor_id);
           *ret_len = strlen(adslAtucPhysStruct.vendor_id);
	   break;

	case ATUC_PHY_VER_NUM_FLAG:
    	   strcpy(sys_info,adslAtucPhysStruct.version_no);
           *ret_len = strlen(adslAtucPhysStruct.version_no);
	   break;
	   
        default:
     	   DEBUGMSGTL(("Unknown MIB variable requested!", "\n"));
           return NULL;
	}

    CLR_FLAG(&adslAtucPhysStruct.flags, flags);
    return (char *) sys_info;
}

#else
/** XXX: return a data pointer to the data for the adslAtucInvSerialNumber
    column and set ret_len to its proper size in bytes. */
char           *
get_adslAtucInvSerialNumber(void *data_context, size_t * ret_len)
{

    struct adslAtucPhysEntry adslAtucPhysStruct;
    memset(&adslAtucPhysStruct,0,sizeof(adslAtucPhysEntry));
    adslAtucPhysStruct.ifIndex= (*(int *)data_context);
    SET_FLAG(&adslAtucPhysStruct.flags,ATUC_PHY_SER_NUM_FLAG);

#ifdef ENABLE_DRIVER_CALL
    adsl_ioctl(Fd, GET_ADSL_ATUC_PHY, (void *)&adslAtucPhysStruct);
#else
    get_adslAtucPhysTable1(Fd, GET_ADSL_ATUC_PHY,(void *)&adslAtucPhysStruct);
#endif
    if (IS_FLAG_SET(&adslAtucPhysStruct.flags, ATUC_PHY_SER_NUM_FLAG) == 0) 
       DEBUGMSGTL(("adslMIB-ATUC", "SerNum CMV fail\n"));
       
    CLR_FLAG(&adslAtucPhysStruct.flags, ATUC_PHY_SER_NUM_FLAG);
    memset(sys_info,0,sizeof(sys_info));
    strcpy(sys_info,adslAtucPhysStruct.serial_no);

    *ret_len = strlen(adslAtucPhysStruct.serial_no);
    return (char *) sys_info;
}

/** XXX: return a data pointer to the data for the adslAtucInvVendorID column
    and set ret_len to its proper size in bytes. */
char           *
get_adslAtucInvVendorID(void *data_context, size_t * ret_len)
{
    struct adslAtucPhysEntry adslAtucPhysStruct;

    memset(&adslAtucPhysStruct,0,sizeof(adslAtucPhysEntry));
    adslAtucPhysStruct.ifIndex= (*(int *)data_context);
    SET_FLAG(&adslAtucPhysStruct.flags,ATUC_PHY_VENDOR_ID_FLAG);

#ifdef ENABLE_DRIVER_CALL
    adsl_ioctl(Fd, GET_ADSL_ATUC_PHY,(void *) &adslAtucPhysStruct);
#else
    get_adslAtucPhysTable1(Fd, GET_ADSL_ATUC_PHY,(void *) &adslAtucPhysStruct);
#endif
    if (IS_FLAG_SET(&adslAtucPhysStruct.flags, ATUC_PHY_VENDOR_ID_FLAG) == 0) 
       DEBUGMSGTL(("adslMIB-ATUC", "VendorID CMV fail\n"));

    CLR_FLAG(&adslAtucPhysStruct.flags, ATUC_PHY_VENDOR_ID_FLAG);

    memset(sys_info,0,sizeof(sys_info));
    strcpy(sys_info,adslAtucPhysStruct.vendor_id);

    *ret_len = strlen(adslAtucPhysStruct.vendor_id);
    return (char *) sys_info;

}

/** XXX: return a data pointer to the data for the adslAtucInvVersionNumber
    column and set ret_len to its proper size in bytes. */

char           *
get_adslAtucInvVersionNumber(void *data_context, size_t * ret_len)
{
    struct adslAtucPhysEntry adslAtucPhysStruct;

    memset(&adslAtucPhysStruct,0,sizeof(adslAtucPhysEntry));
    adslAtucPhysStruct.ifIndex= (*(int *)data_context);
    SET_FLAG(&adslAtucPhysStruct.flags,ATUC_PHY_VER_NUM_FLAG);

#ifdef ENABLE_DRIVER_CALL
    adsl_ioctl(Fd, GET_ADSL_ATUC_PHY,(void *) &adslAtucPhysStruct);
#else
    get_adslAtucPhysTable1(Fd, GET_ADSL_ATUC_PHY,(void *) &adslAtucPhysStruct);
#endif
    if (IS_FLAG_SET(&adslAtucPhysStruct.flags, ATUC_PHY_VER_NUM_FLAG) == 0) 
       DEBUGMSGTL(("adslMIB-ATUC", "VersionNum CMV fail\n"));

    CLR_FLAG(&adslAtucPhysStruct.flags, ATUC_PHY_VER_NUM_FLAG);

    memset(sys_info,0,sizeof(sys_info));
    strcpy(sys_info,adslAtucPhysStruct.version_no);

    *ret_len = strlen(adslAtucPhysStruct.version_no);
    return (char *) sys_info;

}
#endif


/** XXX: return a data pointer to the data for the adslAtucCurrStatus column and
    set ret_len to its proper size in bytes. */

u_int32           *
get_adslAtucCurrStatus(void *data_context, size_t * ret_len)
{
    struct adslAtucPhysEntry adslAtucPhysStruct;

    memset(&adslAtucPhysStruct,0,sizeof(adslAtucPhysEntry));
    adslAtucPhysStruct.ifIndex= (*(int *)data_context);
    SET_FLAG(&adslAtucPhysStruct.flags,ATUC_CURR_STAT_FLAG);

#ifdef ENABLE_DRIVER_CALL
    adsl_ioctl(Fd, GET_ADSL_ATUC_PHY,(void *) &adslAtucPhysStruct);
#else
    get_adslAtucPhysTable1(Fd, GET_ADSL_ATUC_PHY,(void *) &adslAtucPhysStruct);
#endif
    if (IS_FLAG_SET(&adslAtucPhysStruct.flags, ATUC_CURR_STAT_FLAG) == 0) 
       DEBUGMSGTL(("adslMIB-ATUC", "CurrStatus CMV fail\n"));

    CLR_FLAG(&adslAtucPhysStruct.flags, ATUC_CURR_STAT_FLAG);
    long_ret = adslAtucPhysStruct.status;
    *ret_len = sizeof(adslAtucPhysStruct.status);
    return (u_int32 *) (&long_ret);
}

/** XXX: return a data pointer to the data for the adslAtucCurrOutputPwr column
    and set ret_len to its proper size in bytes. */
int           *
get_adslAtucCurrOutputPwr(void *data_context, size_t * ret_len)
{
    struct adslAtucPhysEntry adslAtucPhysStruct;

    memset(&adslAtucPhysStruct,0,sizeof(adslAtucPhysEntry));
    adslAtucPhysStruct.ifIndex= (*(int *)data_context);
    SET_FLAG(&adslAtucPhysStruct.flags,ATUC_CURR_OUT_PWR_FLAG);

#ifdef ENABLE_DRIVER_CALL
    adsl_ioctl(Fd, GET_ADSL_ATUC_PHY,(void *) &adslAtucPhysStruct);
#else
    get_adslAtucPhysTable1(Fd, GET_ADSL_ATUC_PHY,(void *) &adslAtucPhysStruct);
#endif
    if (IS_FLAG_SET(&adslAtucPhysStruct.flags, ATUC_CURR_OUT_PWR_FLAG) == 0) 
       DEBUGMSGTL(("adslMIB-ATUC", "CurrOutPwr CMV fail\n"));

    CLR_FLAG(&adslAtucPhysStruct.flags, ATUC_CURR_OUT_PWR_FLAG);
    long_ret = adslAtucPhysStruct.outputPwr;
    *ret_len = sizeof(adslAtucPhysStruct.outputPwr);
    return (int *) (&long_ret);
}

/** XXX: return a data pointer to the data for the adslAtucCurrAttainableRate
    column and set ret_len to its proper size in bytes. */
u_int32         *
get_adslAtucCurrAttainableRate(void *data_context, size_t * ret_len)
{
    struct adslAtucPhysEntry adslAtucPhysStruct;

    memset(&adslAtucPhysStruct,0,sizeof(adslAtucPhysEntry));
    adslAtucPhysStruct.ifIndex= (*(int *)data_context);
    SET_FLAG(&adslAtucPhysStruct.flags,ATUC_CURR_ATTR_FLAG);

#ifdef ENABLE_DRIVER_CALL
    adsl_ioctl(Fd, GET_ADSL_ATUC_PHY, (void *)&adslAtucPhysStruct);
#else
    get_adslAtucPhysTable1(Fd, GET_ADSL_ATUC_PHY,(void *) &adslAtucPhysStruct);
#endif
    if (IS_FLAG_SET(&adslAtucPhysStruct.flags, ATUC_CURR_ATTR_FLAG) == 0) 
       DEBUGMSGTL(("adslMIB-ATUC", "CurrAttrRate CMV fail\n"));
       
    CLR_FLAG(&adslAtucPhysStruct.flags, ATUC_CURR_ATTR_FLAG);
     long_ret = adslAtucPhysStruct.attainableRate;
    *ret_len = sizeof(adslAtucPhysStruct.attainableRate);
    return (u_int32 *) (&long_ret);
}


#ifndef ENABLE_DRIVER_CALL
int get_adslAtucPhysTable1(int Fd2, int GET_adsl_ATUC_PHY2,
                void *adslAtucPhysTable)
{
    struct adslAtucPhysEntry *adslAtucPhysValue;
    unsigned char temp[12]="Get Value";
    adslAtucPhysValue = (adslAtucPhysEntry *)adslAtucPhysTable;
    strcpy(adslAtucPhysValue->serial_no,temp);
    strcpy(adslAtucPhysValue->vendor_id,temp);
    strcpy(adslAtucPhysValue->version_no,temp);
    adslAtucPhysValue->status=ADSLATUCCURRSTATUS_NODEFECT;
    adslAtucPhysValue->outputPwr=10;
    adslAtucPhysValue->attainableRate=10;

    return 0;
}
#endif
